/*
 * SmartcardioNFC - a library providing a smartcardio implementation for NFC on Android
 *
 * Copyright (C) 2017 gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH <info@gematik.de> +49 30 40041-0
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.gematik.security.test;

import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.Log;

import org.json.JSONException;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.io.IOException;

import javax.smartcardio.CardException;

import de.gematik.security.Utilities.CardFileSystem;
import de.gematik.security.impl.HealthcardContext;
import de.gematik.security.iso7816.Iso7816UnsuccessfulExecutionException;
import de.gematik.security.iso9564.PinBlockException;
import de.gematik.security.nfcCard.NfcCard;

@RunWith(AndroidJUnit4.class)
public class NfcConnectTest extends NfcBaseTest {
    private static final String TAG = NfcConnectTest.class.getName();

    @Test
    public void connectCard() throws CardException, Iso7816UnsuccessfulExecutionException, PinBlockException, IOException, JSONException {
        Log.i(TAG, "connectCard() - only before first test is executed");

        // Injecting the Instrumentation instance is required
        // for your test to run with AndroidJUnitRunner.
        injectInstrumentation(InstrumentationRegistry.getInstrumentation());
        activity = getActivity();

        try {
            Thread nfcThread = new Thread(new NfcControlThread(activity));
            nfcThread.start();

            nfcCard = NfcCard.waitForCardAndConnect(activity, "T=CL");
            activity.setDiscoveredCard(true);

            Log.i(TAG, "Open secure channel");
            healthcardContext = new HealthcardContext(nfcCard.openLogicalChannel(), nfcCard.isExtendedLengthApduSupported());
            if (!healthcardContext.isSMCB()) {
                healthcardContext.secureChannel("123123");
                Log.i(TAG, "Pace established\n");
            }
            healthcardContext.verify("123456");
            Log.i(TAG, "PIN verified\n");

            if (healthcardContext.isEGK()) {
                cfs = CardFileSystem.getEGKInstance();
            } else if (healthcardContext.isHBA()) {
                cfs = CardFileSystem.getHBAInstance();
            } else if (healthcardContext.isSMCB()) {
                cfs = CardFileSystem.getSMCBInstance();
            } else {
                throw new CardException("Inserted card is neither EGK, HBA nor SMC-B.");
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
            throw e;
        }
    }

}
